package se.chalmers.tdv.queue;

import java.lang.reflect.Array;
import java.util.ArrayList;
import java.util.Random;

import se.chalmers.tdv.minimize.Tuple;
import se.chalmers.tdv.testing.Property;
import se.chalmers.tdv.testing.TestCase;
import se.chalmers.tdv.testing.TestRunner;
import se.chalmers.tdv.queue.QueueAction.*;

@SuppressWarnings("unchecked")
public class QueueTestSuite {
	// set this to true  to generate code to past into a file to use the debugger
	static final boolean PRINT_CODE = false;
	static final int MAX_PREFIX_LENGTH = 10;
	static final int MAX_QUEUE_SIZE = 4;
	static final int MAX_POSTFIX_LENGTH = 10;
	static final int MAX_NR_TESTS = 10000;
	static final int MAX_RANDOM_INT = 10;

	static Integer A = 10;
	static Integer B = 12;
	static Integer C = 1;
	
	// Action generator methods:
	
	public static QueueAction isEmpty() {
		return new QueueAction(ActionType.ISEMPTY, 0);
	}

	public static QueueAction add(Integer value) {
		return new QueueAction(ActionType.ADD, value);
	}

	public static QueueAction front() {
		return new QueueAction(ActionType.FRONT, 0);
	}

	public static QueueAction remove() {
		return new QueueAction(ActionType.REMOVE, 0);
	}

	public static QueueAction output(Integer value) {
		return new QueueAction(ActionType.OUTPUT, value);
	}
	
	
	
	static final Property<QueueAction> frontEmpty = new Property<>("frontEmpty", false, true, 
			new QueueAction[]{ front()}, 
			new QueueAction[]{ output(null)});
	/** front() <==> output(null) (on empty list) */
	
	static final Property<QueueAction> front  = null;
			/** add(A), front <==> add(A), out(A) (on empty queue) */
	
	static final Property<QueueAction> frontAddTwo =  null;
	/** add(A), add(B), front <==> add(A), front, add(B) */

	static final Property<QueueAction> remove = null; // Fix me!
	/** add(A), remove <==> no action (on empty queue) */

	static final Property<QueueAction> removeAddTwo = null; // Fix me!
	/** add(A), add(B), remove <==> add(A), remove, add(B) */

	// Java generic and arrays = headache
	static final Object[] allPropertiesObjects = new Object[]
			{ frontEmpty, front, frontAddTwo, remove, removeAddTwo };
	
	static final Property<QueueAction>[] allProperties;
	
	static {
		allProperties = (Property<QueueAction>[]) Array.newInstance(Property.class, allPropertiesObjects.length);
		for(int i = 0 ; i < allProperties.length; i++){
			allProperties[i] = (Property<QueueAction>) allPropertiesObjects[i];
		}
	}
	

	static TestCase<QueueAction> generate(Property<QueueAction> property){
		QueueAction[] prefix  = property.prefix ? QueueAction.randomActions(MAX_PREFIX_LENGTH)   : new QueueAction[]{};
		QueueAction[] postfix = property.postfix ? QueueAction.randomActions(MAX_POSTFIX_LENGTH) : new QueueAction[]{};
		return new TestCase<>(prefix, property.commandsLeft, property.commandsRight, postfix);
	}
	
	static void runTest(Property<QueueAction> property){
		
		Random rand = new Random();
		
		System.out.println("Testing property: " + property.name);
		for(int i = 0 ; i < MAX_NR_TESTS; i++){
			int queueSize = rand.nextInt(MAX_QUEUE_SIZE) + 1;
			TestRunner<QueueAction, Tuple<Queue,ArrayList<Integer>>> runner = new TestRunner<>(QueueAction.class,new QueueInterpret(queueSize));
			A = new Integer(rand.nextInt(MAX_RANDOM_INT));
			B = new Integer(rand.nextInt(MAX_RANDOM_INT));
			C = new Integer(rand.nextInt(MAX_RANDOM_INT));
			if(!runner.runTestPrint(PRINT_CODE,generate(property))){
				return;
			}
		}
		System.out.println(MAX_NR_TESTS + " test succeeded");
	}
	
	/** Runs until all test are successful or a bug is found. */
	static void runAllTests() {
		for(Property<QueueAction> prop : allProperties){
			if(prop!=null){
				runTest(prop);
			}
		}
	}
	
	public static void main(String[] args){
		runAllTests();
	}
	
	

	
}

